function [optim_pars, results] = ...
        do_registration(im, template, ...
                        init_ranges, ...
                        init_samples_per_shorter_side, ...
                        fnc_min, ...
                        termination_delta_search_space)

% structure for logging the progress of optimization:
results = struct('optim_pars', cell(1), ...
                 'optim_subimage', cell(1), ...
                 'ranges', cell(1)); 

ranges = init_ranges; 

% keep original images:
orig = struct('im', im, ...
              'template', template); 

shorter_side = min(size(template)); 

% compute delta:
% (delta is equivalent to step size in pixel units)
delta_ = shorter_side/init_samples_per_shorter_side; 

delta = struct('search_space', delta_, ...
               'sampling', delta_); 

template_size = size(template); 

iteration = 0; 

% IMPLEMENT: change the termination condition
while iteration < 1

    iteration = iteration+1; 
    
    sigma = delta_sampling_to_sigma(delta.sampling); 

    % apply low pass filter:
    im = lowpass(orig.im, sigma); 
    template = lowpass(orig.template, sigma); 

    % compute sampling grid: 
    [xgrid, ygrid] = compute_sampling_grid(delta.sampling, template_size);

    % compute search space: 
    search_space = compute_search_space(delta.search_space, ...
                                        template_size, ...
                                        ranges); 
    
    [optim_par_idcs, optim_pars, optim_subimage, optim_val] = ...
        find_best(im, template, search_space, xgrid, ygrid, fnc_min); 
    
    % log the progress to results: 
    results.optim_pars{iteration} = optim_pars; 
    results.optim_subimage{iteration} = optim_subimage; 
    results.ranges{iteration} = ranges; 
    
    % IMPLEMENT: define new ranges to shrink the search space 
    % ranges.rot = 
    % ranges.xshift = ... 
    % ranges.yshift = ... 

    % halve delta for search space: 
    delta.search_space = delta.search_space/2;
    
    % do the same with delta for sampling but keep 
    % it higher than a constant (here 1.5) to be faster: 
    delta.sampling = max( delta.sampling/2, 1.5); 

end

% done!
